@@ -8,6 +8,7 @@
 package Portal::UserProperties::User;
 use strict;
 use Portal::AppState;
+use Portal::Forms::User;
 use Portal::UserProperties::Objects::Menu;
 use Portal::XML::ConfigParser;
 use Portal::XML::ConfigObject;
@@ -23,7 +24,7 @@ $VERSION = '0.05';
 
 # global variables.
 # modify to represent the commands this state provides.
-my %commands = ( display => "User Properties", passwd => "Change User Password", preferences => "User Preferences", );
+my %commands = ( display => "View User Info", editUser => "Edit User Info", changePassword => "Change User Password", preferences => "User Preferences", );
 
 =head1 NAME
 
@@ -33,7 +34,7 @@ User - Object used to build a User Object Class.
 
   use Portal::UserProperties::User;
   my $obj = Portal::UserProperties::User->new;
-  if ($obj->didErrorOccur())
+  if ($obj->error())
   {
     die $obj->errorMessage();
   }
@@ -62,7 +63,7 @@ sub new
   my $self = $class->SUPER::new(@_);
   my %args = ( @_ );
 
-  if ($self->didErrorOccur)
+  if ($self->error)
   {
     $self->prefixError();
     return $self;
@@ -70,6 +71,21 @@ sub new
 
   # instantiate anything unique to this module
   $self->{commands} = \%commands;
+  $self->{state} = "User";
+  
+  my $arguments = $self->arguments();
+  eval "\$self->{userForms} = Portal::Forms::User->new($arguments);";
+  if ($@)
+  {
+    $self->error($@);
+    return $self;
+  }
+  if ($self->{userForms}->error)
+  {
+    $self->error($self->{userForms}->errorMessage);
+    return $self;
+  }
+  
 
   # do validation
   if (!$self->isValid)
@@ -117,7 +133,7 @@ sub isValid
 
   if ($self->numInvalid() > 0 || $self->numMissing() > 0)
   {
-    $self->postfixError($self->genErrorString("all"));
+    $self->error($self->genErrorString("all"));
     return 0;
   }
 
@@ -131,122 +147,166 @@ sub c_display
 {
   my $self = shift;
   my $doc = $self->setupCSS();
-  if ($self->didErrorOccur)
+  if ($self->error)
   {
     $self->prefixError();
     return undef;
   }
 
-  if (! exists $self->{input}->{update})
+  # start the page display overall with the menus.
+  $doc = $self->displayMenu(doc => $doc, command => "display");
+  if ($self->error)
   {
-    $doc = $self->displayUserProperties(doc => $doc);
-    if ($self->didErrorOccur)
+    $self->prefixError();
+    return undef;
+  }
+
+  # call the Portal::Forms::User module to do the core display, validation and update work
+  my %output = $self->{userForms}->viewUser(user => $self->{userObj}, companyObj => $self->{companyObj});
+  if ($self->{userForms}->error)
+  {
+    $self->error($self->{userForms}->errorMessage);
+    return undef;
+  }
+  
+  # now display the output to the user.
+  $doc->print(%output);
+  
+  # finish off the screen, closing out what the displayMenu() method started.
+  $doc = $self->closeMenu(doc => $doc);
+
+  return $doc;
+}
+
+=item HTMLObject c_editUser
+
+=cut
+sub c_editUser
+{
+  my $self = shift;
+  my $doc = $self->setupCSS();
+  if ($self->error)
+  {
+    $self->prefixError();
+    return undef;
+  }
+  my $successEditPhrase = $self->langObj->map("successUserEdit");
+
+  my $url = $self->{methods}->createBaseURL(type => "App", linkType => "cgi", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});
+  if ($self->{methods}->error)
+  {
+    $self->error($self->{methods}->errorMessage);
+    return undef;
+  }
+
+  # start the page display overall with the menus.
+  $doc = $self->displayMenu(doc => $doc, command => "editUser");
+  if ($self->error)
+  {
+    $self->prefixError();
+    return undef;
+  }
+
+  # call the Portal::Forms::User module to do the core display, validation and update work
+  my @result = $self->{userForms}->editUser(user => $self->{userObj},
+      url => $url, app => $self->{app}, state => $self->{state});
+      
+  if ($self->{userForms}->error)
+  {
+    $self->error($self->{userForms}->errorMessage);
+    return undef;
+  }
+  
+  # now display the output to the user.
+  if ($result[0] == 2)
+  {
+    $doc->print(%{$result[1]});
+    $doc->setOnload(code => $result[3]);
+  }
+  elsif ($result[0] == 1)
+  {
+    $self->{methods}->displayMessage(doc => $doc, type => "status", message => $successEditPhrase,
+                                     langObj => $self->{langObj}, break => "below");
+    if ($self->{methods}->error)
     {
-      $self->prefixError();
+      $self->error($self->{methods}->errorMessage);
       return undef;
     }
+    
+    # update the session for our userObj.
+    $self->{portalSession}->{store}->{userObj} = $result[2];
+    $self->{portalSession}->{store}->{changed} = 1;
   }
-  else  # validate and update the users properties.
+  
+  # finish off the screen, closing out what the displayMenu() method started.
+  if ($result[0] != 0)
   {
+    $doc = $self->closeMenu(doc => $doc);
   }
 
   return $doc;
 }
 
-=item HTMLObject c_passwd
+=item HTMLObject c_changePassword
 
 =cut
-sub c_passwd
+sub c_changePassword
 {
   my $self = shift;
   my $doc = $self->setupCSS();
-  if ($self->didErrorOccur)
+  if ($self->error)
   {
     $self->prefixError();
     return undef;
   }
-  my $passwordLengthPhrase = $self->langObj->map("passwordLength");
-  my $passwordNotEqualPhrase = $self->langObj->map("passwordNotEqual");
-  my $passwordErrorPhrase = $self->langObj->map("passwordError");
   my $successPasswordPhrase = $self->langObj->map("successPassword");
 
-  if (! exists $self->{input}->{update})
+  my $url = $self->{methods}->createBaseURL(type => "App", linkType => "cgi", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});
+  if ($self->{methods}->error)
   {
-    $doc = $self->displayPasswordScreen(doc => $doc);
-    if ($self->didErrorOccur)
+    $self->error($self->{methods}->errorMessage);
+    return undef;
+  }
+
+  # start the page display overall with the menus.
+  $doc = $self->displayMenu(doc => $doc, command => "changePassword");
+  if ($self->error)
+  {
+    $self->prefixError();
+    return undef;
+  }
+
+  # call the Portal::Forms::User module to do the core display, validation and update work
+  my @result = $self->{userForms}->changePassword(user => $self->{userObj},
+      url => $url, app => $self->{app}, state => $self->{state});
+      
+  if ($self->{userForms}->error)
+  {
+    $self->error($self->{userForms}->errorMessage);
+    return undef;
+  }
+  
+  # now display the output to the user.
+  if ($result[0] == 2)
+  {
+    $doc->print(%{$result[1]});
+    $doc->setOnload(code => $result[2]);
+  }
+  elsif ($result[0] == 1)
+  {
+    $self->{methods}->displayMessage(doc => $doc, type => "status", message => $successPasswordPhrase,
+                                     langObj => $self->{langObj}, break => "below");
+    if ($self->{methods}->error)
     {
-      $self->prefixError();
+      $self->error($self->{methods}->errorMessage);
       return undef;
     }
   }
-  else  # validate and update the password.
+  
+  # finish off the screen, closing out what the displayMenu() method started.
+  if ($result[0] != 0)
   {
-    # make sure they specified original, password1 and password2
-    my $original = $self->{input}->{original};
-    my $password1 = $self->{input}->{password1};
-    my $password2 = $self->{input}->{password2};
-
-    my $errorString = "";
-    if (length $original < 6)
-    {
-      $errorString .= sprintf($passwordLengthPhrase, "original") . "<br>\n";
-    }
-    else
-    {
-      # actually validate the original password.
-      my $result = $self->{authObj}->isUserValid(uname => $self->{userObj}->{uname}, password => $original);
-      if ($result == 0)
-      {
-        $errorString .= sprintf($passwordNotEqualPhrase, "original") . "<br>\n";
-      }
-      elsif ($result < 0)
-      {
-        $errorString .= sprintf($passwordErrorPhrase, "$result") . "<br>\n";
-      }
-    }
-    if (length $password1 < 6)
-    {
-      $errorString .= sprintf($passwordLengthPhrase, "") . "<br>\n";
-    }
-    if ($password1 ne $password2)
-    {
-      $errorString .= sprintf($passwordNotEqualPhrase, "") . "<br>\n";
-    }
-
-    if (length $errorString > 0)
-    {
-      $doc = $self->displayPasswordScreen(doc => $doc, error => $errorString);
-      if ($self->didErrorOccur)
-      {
-        $self->prefixError();
-        return undef;
-      }
-    }
-    else
-    {
-      # now we update the userObj with the new password and signal that it needs to be encrypted
-      # and then update the users data.
-
-      $self->{userObj}->{password} = $password1;
-      $self->{userObj}->{passCrypted} = "false";
-
-      my $result = $self->{authObj}->updateUser(userObj => $self->{userObj});
-      if ($self->{authObj}->didErrorOccur)
-      {
-        $doc = $self->displayPasswordScreen(doc => $doc, error => $self->{authObj}->errorMessage);
-        if ($self->didErrorOccur)
-        {
-          $self->prefixError();
-          return undef;
-        }
-      }
-      $doc = $self->displayPasswordScreen(doc => $doc, status => $successPasswordPhrase);
-      if ($self->didErrorOccur)
-      {
-        $self->prefixError();
-        return undef;
-      }
-    }
+    $doc = $self->closeMenu(doc => $doc);
   }
 
   return $doc;
@@ -259,7 +319,7 @@ sub c_preferences
 {
   my $self = shift;
   my $doc = $self->setupCSS();
-  if ($self->didErrorOccur)
+  if ($self->error)
   {
     $self->prefixError();
     return undef;
@@ -269,7 +329,7 @@ sub c_preferences
   if (! exists $self->{input}->{update})
   {
     $doc = $self->displayUserPreferences(doc => $doc);
-    if ($self->didErrorOccur)
+    if ($self->error)
     {
       $self->prefixError();
       return undef;
@@ -287,15 +347,15 @@ sub c_preferences
     # do validation
     my $errorString = "";
     my @systemColorSchemes = $self->{applicationObj}->getAvailableColorSchemes(userId => "-1");
-    if ($self->{applicationObj}->didErrorOccur)
+    if ($self->{applicationObj}->error)
     {
-      $self->setError(errorString => $self->{applicationObj}->errorMessage);
+      $self->error($self->{applicationObj}->errorMessage);
       return undef;
     }
     my @userColorSchemes = $self->{applicationObj}->getAvailableColorSchemes(userId => $self->{userObj}->{id});
-    if ($self->{applicationObj}->didErrorOccur)
+    if ($self->{applicationObj}->error)
     {
-      $self->setError(errorString => $self->{applicationObj}->errorMessage);
+      $self->error($self->{applicationObj}->errorMessage);
       return undef;
     }
     my $systemColorScheme = grep($colorScheme, @systemColorSchemes);
@@ -305,7 +365,7 @@ sub c_preferences
     {
       $errorString .= sprintf($invalidArgument, "colorScheme", $colorScheme);
     }
-    if ($dynamicContent !~ /^(true|false)$/)
+    if ($dynamicContent !~ /^(1|0)$/)
     {
       $errorString .= sprintf($invalidArgument, "dynamicContent", $dynamicContent);
     }
@@ -329,7 +389,7 @@ sub c_preferences
     if (length $errorString > 0)
     {
       $doc = $self->displayUserPreferences(doc => $doc, error => $errorString);
-      if ($self->didErrorOccur)
+      if ($self->error)
       {
         $self->prefixError();
         return undef;
@@ -351,7 +411,7 @@ sub c_preferences
           if ($@)
           {
             $doc = $self->displayUserPreferences(doc => $doc, error => $@);
-            if ($self->didErrorOccur)
+            if ($self->error)
             {
               $self->prefixError();
               return undef;
@@ -369,7 +429,7 @@ sub c_preferences
           if ($@)
           {
             $doc = $self->displayUserPreferences(doc => $doc, error => $@);
-            if ($self->didErrorOccur)
+            if ($self->error)
             {
               $self->prefixError();
               return undef;
@@ -379,7 +439,7 @@ sub c_preferences
           if (length $configObjStr == 0)
           {
             $doc = $self->displayUserPreferences(doc => $doc, error => $self->{langObj}->map("noConfigXMLGenerated"));
-            if ($self->didErrorOccur)
+            if ($self->error)
             {
               $self->prefixError();
               return undef;
@@ -392,10 +452,10 @@ sub c_preferences
         my $prefObj = Portal::Objects::UserPreferenceObject->new(id => $self->{userObj}->{id},
                       app => "Portal", module => $module, name => "$name",
                       value => ($name ne "dynamicContentConfig" ? $self->{input}->{$name} : $configObjStr), langObj => $self->{langObj});
-        if ($prefObj->didErrorOccur)
+        if ($prefObj->error)
         {
           $doc = $self->displayUserPreferences(doc => $doc, error => $prefObj->errorMessage);
-          if ($self->didErrorOccur)
+          if ($self->error)
           {
             $self->prefixError();
             return undef;
@@ -403,10 +463,10 @@ sub c_preferences
           return $doc;
         }
         my $result = $self->{authObj}->updateUserPreference(userPreferenceObj => $prefObj);
-        if ($self->{authObj}->didErrorOccur)
+        if ($self->{authObj}->error)
         {
           $doc = $self->displayUserPreferences(doc => $doc, error => $self->{authObj}->errorMessage);
-          if ($self->didErrorOccur)
+          if ($self->error)
           {
             $self->prefixError();
             return undef;
@@ -416,10 +476,10 @@ sub c_preferences
         if ($result == -1)
         { # have to create it
           $result = $self->{authObj}->createUserPreference(userPreferenceObj => $prefObj);
-          if ($self->{authObj}->didErrorOccur)
+          if ($self->{authObj}->error)
           {
             $doc = $self->displayUserPreferences(doc => $doc, error => $self->{authObj}->errorMessage);
-            if ($self->didErrorOccur)
+            if ($self->error)
             {
               $self->prefixError();
               return undef;
@@ -429,7 +489,7 @@ sub c_preferences
           if ($result == -1)
           { # already existed!  don't know how!
             $doc = $self->displayUserPreferences(doc => $doc, error => sprintf($self->{langObj}->map("createUserPreferenceNowExists"), $prefObj->print));
-            if ($self->didErrorOccur)
+            if ($self->error)
             {
               $self->prefixError();
               return undef;
@@ -439,7 +499,7 @@ sub c_preferences
           elsif ($result == -2)
           { # app doesn't exist
             $doc = $self->displayUserPreferences(doc => $doc, error => sprintf($self->{langObj}->map("createUserPreferenceAppNotExist"), $prefObj->print));
-            if ($self->didErrorOccur)
+            if ($self->error)
             {
               $self->prefixError();
               return undef;
@@ -453,7 +513,7 @@ sub c_preferences
         elsif ($result == -2)
         { # app doesn't exist
           $doc = $self->displayUserPreferences(doc => $doc, error => sprintf($self->langObj->map("updateUserPreferenceAppNotExist"), $prefObj->print));
-          if ($self->didErrorOccur)
+          if ($self->error)
           {
             $self->prefixError();
             return undef;
@@ -466,7 +526,7 @@ sub c_preferences
       }
 
       $doc = $self->displayUserPreferences(doc => $doc, status => $successPreferencesPhrase);
-      if ($self->didErrorOccur)
+      if ($self->error)
       {
         $self->prefixError();
         return undef;
@@ -477,140 +537,6 @@ sub c_preferences
   return $doc;
 }
 
-sub displayUserProperties
-{
-  my $self = shift;
-  my %args = ( doc => undef, error => "", status => "", @_ );
-  my $doc = $args{doc};
-  my $error = $args{error};
-  my $status = $args{status};
-
-  my $changePasswordPhrase = $self->langObj->map("changePassword");
-  my $passwordPhrase = $self->langObj->map("password");
-  my $repeatPhrase = $self->langObj->map("repeat");
-
-  my $url = $self->{methods}->createBaseURL(type => "App", linkType => "cgi", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});
-  if ($self->{methods}->didErrorOccur)
-  {
-    $self->setError(errorString => $self->{methods}->errorMessage);
-    return undef;
-  }
-
-  $doc = $self->displayMenu(doc => $doc, command => "passwd");
-  if ($self->didErrorOccur)
-  {
-    $self->prefixError();
-    return $doc;
-  }
-
-  if ($error)
-  {
-    $self->{methods}->displayMessage(doc => $doc, type => "error", message => $error,
-                                     langObj => $self->{langObj}, break => "below");
-    if ($self->{methods}->didErrorOccur)
-    {
-      $self->setError(errorString => $self->{methods}->errorMessage);
-      return undef;
-    }
-  }
-  if ($status)
-  {
-    $self->{methods}->displayMessage(doc => $doc, type => "status", message => $status,
-                                     langObj => $self->{langObj}, break => "below");
-    if ($self->{methods}->didErrorOccur)
-    {
-      $self->setError(errorString => $self->{methods}->errorMessage);
-      return undef;
-    }
-  }
-
-  return $doc;
-}
-
-sub displayPasswordScreen
-{
-  my $self = shift;
-  my %args = ( doc => undef, error => "", status => "", @_ );
-  my $doc = $args{doc};
-  my $error = $args{error};
-  my $status = $args{status};
-
-  my $changePasswordPhrase = $self->langObj->map("changePassword");
-  my $passwordPhrase = $self->langObj->map("password");
-  my $repeatPhrase = $self->langObj->map("repeat");
-  my $originalPhrase = $self->langObj->map("original");
-  my $newPhrase = $self->langObj->map("new");
-
-  my $url = $self->{methods}->createBaseURL(type => "App", linkType => "cgi", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});
-  if ($self->{methods}->didErrorOccur)
-  {
-    $self->setError(errorString => $self->{methods}->errorMessage);
-    return undef;
-  }
-
-  $doc = $self->displayMenu(doc => $doc, command => "passwd");
-  if ($self->didErrorOccur)
-  {
-    $self->prefixError();
-    return $doc;
-  }
-
-  if ($error)
-  {
-    $self->{methods}->displayMessage(doc => $doc, type => "error", message => $error,
-                                     langObj => $self->{langObj}, break => "below");
-    if ($self->{methods}->didErrorOccur)
-    {
-      $self->setError(errorString => $self->{methods}->errorMessage);
-      return undef;
-    }
-  }
-  if ($status)
-  {
-    $self->{methods}->displayMessage(doc => $doc, type => "status", message => $status,
-                                     langObj => $self->{langObj}, break => "below");
-    if ($self->{methods}->didErrorOccur)
-    {
-      $self->setError(errorString => $self->{methods}->errorMessage);
-      return undef;
-    }
-  }
-
-  # generate the form to let them specify their new password in.
-  $doc->print(<<"END_OF_CODE");
-$changePasswordPhrase
-<br>
-<form name="passwds" method="post" action="$url">
-  <input type="hidden" name="app" value="UserProperties">
-  <input type="hidden" name="state" value="User">
-  <input type="hidden" name="command" value="passwd">
-  <input type="hidden" name="update" value="1">
-  <table border="0" cellpadding="2" cellspacing="0" class="edit">
-    <tr>
-      <td align="right">$originalPhrase $passwordPhrase</td>
-      <td align="left"><input type="password" name="original" value=""></td>
-    </tr>
-    <tr>
-      <td align="right">$newPhrase $passwordPhrase</td>
-      <td align="left"><input type="password" name="password1" value=""></td>
-    </tr>
-    <tr>
-      <td align="right">$repeatPhrase</td>
-      <td align="left"><input type="password" name="password2" value=""></td>
-    </tr>
-  </table>
-  <input type="submit" name="update" value="$changePasswordPhrase">
-</form>
-    </td>
-  </tr>
-</table>
-END_OF_CODE
-
-  $doc->setOnload(code => "document.passwds.original.focus();");
-
-  return $doc;
-}
-
 sub displayUserPreferences
 {
   my $self = shift;
@@ -634,7 +560,7 @@ sub displayUserPreferences
   eval { $parserObj = Portal::XML::ConfigParser->new(langObj => $self->{langObj}); };
   if ($@)
   {
-    $self->setError(errorString => $@);
+    $self->error($@);
     return $doc;
   }
 
@@ -647,14 +573,14 @@ sub displayUserPreferences
     {
       my $prefNameDefault = "Portal_" . $prefModule . "_" . $prefName . "_Default";
       my $prefValueDefault = $self->{methods}->getConfigValue(name => $prefNameDefault, portalDB => $self->{applicationObj}->{portalDB});
-      if ($self->{methods}->didErrorOccur)
+      if ($self->{methods}->error)
       {
-        $self->setError(errorString => $self->{methods}->errorMessage);
+        $self->error($self->{methods}->errorMessage);
         return $doc;
       }
       if (!defined $prefValueDefault)
       {
-        $self->setError(errorString => sprintf($self->{langObj}->map("configEntryDoesNotExistUpgradeDatabase"), $prefNameDefault));
+        $self->error(sprintf($self->{langObj}->map("configEntryDoesNotExistUpgradeDatabase"), $prefNameDefault));
         return $doc;
       }
       if ($prefName eq "dynamicContent_Config" && $prefModule eq "Desktop")
@@ -664,7 +590,7 @@ sub displayUserPreferences
         eval { $configObj = $parserObj->parse(string => $prefValueDefault, module => "DynamicContentConfig", version => "1.0"); };
         if ($@)
         {
-          $self->setError(errorString => sprintf($self->{langObj}->map("parseFailed"), $@));
+          $self->error(sprintf($self->{langObj}->map("parseFailed"), $@));
           return $doc;
         }
         foreach my $name (keys %{$configObj->{settings}})
@@ -682,9 +608,9 @@ sub displayUserPreferences
   # pull in the Portal preferences for the user.
   my @userPreferences = $self->{authObj}->getUserPreferences(userId => $self->{userObj}->{id},
                         app => "Portal");
-  if ($self->{authObj}->didErrorOccur)
+  if ($self->{authObj}->error)
   {
-    $self->setError(errorString => $self->{authObj}->errorMessage);
+    $self->error($self->{authObj}->errorMessage);
     return $doc;
   }
   my %preferences = ();  # store the preferences info that we get from the system.
@@ -697,7 +623,7 @@ sub displayUserPreferences
       eval { $configObj = $parserObj->parse(string => $prefObj->{value}, module => "DynamicContentConfig", version => "1.0"); };
       if ($@)
       {
-        $self->setError(errorString => sprintf($self->{langObj}->map("parseFailed"), $@));
+        $self->error(sprintf($self->{langObj}->map("parseFailed"), $@));
         return $doc;
       }
       foreach my $name (keys %{$configObj->{settings}})
@@ -722,14 +648,14 @@ sub displayUserPreferences
   }
 
   my $url = $self->{methods}->createBaseURL(type => "App", linkType => "cgi", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});
-  if ($self->{methods}->didErrorOccur)
+  if ($self->{methods}->error)
   {
-    $self->setError(errorString => $self->{methods}->errorMessage);
+    $self->error($self->{methods}->errorMessage);
     return undef;
   }
 
   $doc = $self->displayMenu(doc => $doc, command => "preferences");
-  if ($self->didErrorOccur)
+  if ($self->error)
   {
     $self->prefixError();
     return $doc;
@@ -739,9 +665,9 @@ sub displayUserPreferences
   {
     $self->{methods}->displayMessage(doc => $doc, type => "error", message => $error,
                                      langObj => $self->{langObj}, break => "below");
-    if ($self->{methods}->didErrorOccur)
+    if ($self->{methods}->error)
     {
-      $self->setError(errorString => $self->{methods}->errorMessage);
+      $self->error($self->{methods}->errorMessage);
       return undef;
     }
   }
@@ -749,9 +675,9 @@ sub displayUserPreferences
   {
     $self->{methods}->displayMessage(doc => $doc, type => "status", message => $status,
                                      langObj => $self->{langObj}, break => "below");
-    if ($self->{methods}->didErrorOccur)
+    if ($self->{methods}->error)
     {
-      $self->setError(errorString => $self->{methods}->errorMessage);
+      $self->error($self->{methods}->errorMessage);
       return undef;
     }
   }
@@ -761,8 +687,8 @@ sub displayUserPreferences
 $editPhrase $userPreferencesPhrase
 <br>
 <form name="preferences" method="post" action="$url">
-  <input type="hidden" name="app" value="UserProperties">
-  <input type="hidden" name="state" value="User">
+  <input type="hidden" name="app" value="$self->{app}">
+  <input type="hidden" name="state" value="$self->{state}">
   <input type="hidden" name="command" value="preferences">
   <input type="hidden" name="update" value="1">
   <table border="0" cellpadding="2" cellspacing="0" class="edit">
@@ -772,15 +698,15 @@ $editPhrase $userPreferencesPhrase
         <select name="colorScheme">
 END_OF_CODE
     my @systemColorSchemes = $self->{applicationObj}->getAvailableColorSchemes(userId => "-1");
-    if ($self->{applicationObj}->didErrorOccur)
+    if ($self->{applicationObj}->error)
     {
-      $self->setError(errorString => $self->{applicationObj}->errorMessage);
+      $self->error($self->{applicationObj}->errorMessage);
       return undef;
     }
     my @userColorSchemes = $self->{applicationObj}->getAvailableColorSchemes(userId => $self->{userObj}->{id});
-    if ($self->{applicationObj}->didErrorOccur)
+    if ($self->{applicationObj}->error)
     {
-      $self->setError(errorString => $self->{applicationObj}->errorMessage);
+      $self->error($self->{applicationObj}->errorMessage);
       return undef;
     }
     foreach my $option (@systemColorSchemes, @userColorSchemes)
@@ -797,10 +723,10 @@ END_OF_CODE
       <td align="left">
         <select name="dynamicContent">
 END_OF_CODE
-    foreach my $option ("true", "false")
+    foreach my $option ("1", "0")
     {
       my $selected = ($preferences{Desktop}{dynamicContent} eq "$option" ? " selected" : "");
-      $doc->print("          <option value=\"$option\"$selected>$option\n");
+      $doc->print("          <option value=\"$option\"$selected>" . ($option ? "true" : "false") . "\n");
     }
   $doc->print(<<"END_OF_CODE");
         </select>
@@ -865,40 +791,40 @@ sub displayMenu
 
   # create the help JS function.
   $doc = $self->{methods}->displayJSHelper(doc => $doc, type => "help", langObj => $self->{langObj});
-  if ($self->{methods}->didErrorOccur)
+  if ($self->{methods}->error)
   {
-    $self->setError(errorString => $self->{methods}->errorMessage);
+    $self->error($self->{methods}->errorMessage);
     return undef;
   }
 
   # create the validateClose JS function.
   $doc = $self->{methods}->displayJSHelper(doc => $doc, type => "closeApp", langObj => $self->{langObj});
-  if ($self->{methods}->didErrorOccur)
+  if ($self->{methods}->error)
   {
-    $self->setError(errorString => $self->{methods}->errorMessage);
+    $self->error($self->{methods}->errorMessage);
     return undef;
   }
 
   my $menuObj = Portal::UserProperties::Objects::Menu->new(methods => $self->{methods},
                 sessionObj => $self->{sessionObj}, langObj => $self->{langObj}, userObj => $self->{userObj},
                 portalSession => $self->{portalSession});
-  if ($menuObj->didErrorOccur)
+  if ($menuObj->error)
   {
-    $self->setError(errorString => $menuObj->errorMessage);
+    $self->error($menuObj->errorMessage);
     return undef;
   }
 
-  my $menuString = $menuObj->print(location => "menubar", state => "User", command => $command, doc => $doc);
-  if ($menuObj->didErrorOccur)
+  my $menuString = $menuObj->print(location => "menubar", state => $self->{state}, command => $command, doc => $doc);
+  if ($menuObj->error)
   {
-    $self->setError(errorString => $menuObj->errorMessage);
+    $self->error($menuObj->errorMessage);
     return undef;
   }
 
-  my $sidemenuString = $menuObj->print(location => "sidemenu", state => "User", command => $command, doc => $doc);
-  if ($menuObj->didErrorOccur)
+  my $sidemenuString = $menuObj->print(location => "sidemenu", state => $self->{state}, command => $command, doc => $doc);
+  if ($menuObj->error)
   {
-    $self->setError(errorString => $menuObj->errorMessage);
+    $self->error($menuObj->errorMessage);
     return undef;
   }
 
@@ -916,6 +842,21 @@ END_OF_CODE
   return $doc;
 }
 
+sub closeMenu
+{
+  my $self = shift;
+  my %args = ( doc => undef, @_ );
+  my $doc = $args{doc};
+  
+  $doc->print(<<"END_OF_FORM");
+    </td>
+  </tr>
+</table>
+END_OF_FORM
+
+  return $doc;  
+}
+
 1;
 __END__
 
